@@ -22,6 +22,10 @@
 #include <stdlib.h> // setenv
 #include <sys/prctl.h> // prctl(), PR_SET_PDEATHSIG
 
+#undef  ZYPP_BASE_LOGGER_LOGGROUP
+#define ZYPP_BASE_LOGGER_LOGGROUP "zypp::exec"
+
+
 zyppng::AbstractDirectSpawnEngine::~AbstractDirectSpawnEngine()
 {
   if ( AbstractDirectSpawnEngine::isRunning() ) {
@@ -63,7 +67,7 @@ void zyppng::AbstractDirectSpawnEngine::mapExtraFds ( int controlFd )
   int nextBackupFd = lastFdToKeep + 1; //this we will use to count the fds upwards
   std::vector<int> safeFds;
   for ( auto fd : _mapFds ) {
-    // if the fds are bigger than the last one we will map to its safe
+    // If the fds are larger than the last one we will map to, it is safe.
     if ( fd > lastFdToKeep ) {
       safeFds.push_back( fd );
     } else {
@@ -142,28 +146,19 @@ bool zyppng::ForkSpawnEngine::start( const char * const *argv, int stdin_fd, int
   _executedCommand.clear();
   _args.clear();
 
-  const char * chdirTo = nullptr;
-
   if ( !argv || !argv[0] ) {
     _execError = _("Invalid spawn arguments given.");
     _exitStatus = 128;
     return false;
   }
 
-  if ( !_chroot.empty() )
-  {
-    const auto chroot = _chroot.c_str();
-    if ( chroot[0] == '\0' )
-    {
-      _chroot = zypp::Pathname();	// ignore empty _chroot
-    }
-    else if ( chroot[0] == '/' && chroot[1] == '\0' )
-    {
-      // If _chroot is '/' do not chroot, but chdir to '/'
-      // unless arglist defines another dir.
-      chdirTo = "/";
-      _chroot = zypp::Pathname();
-    }
+  const char * chdirTo = nullptr;
+
+  if ( _chroot == "/" ) {
+    // If _chroot is '/' do not chroot, but chdir to '/'
+    // unless arglist defines another dir.
+    chdirTo = "/";
+    _chroot = zypp::Pathname();
   }
 
   if ( !_workingDirectory.empty() )
@@ -353,6 +348,7 @@ bool zyppng::ForkSpawnEngine::start( const char * const *argv, int stdin_fd, int
           _execError = zypp::str::form( _("Can't exec '%s', unexpected error."), _args[0].c_str() );
           break;
       }
+      ERR << "pid " << _pid << " launch failed: " << _execError << std::endl;
 
       // reap child and collect exit code
       isRunning( true );
@@ -400,19 +396,11 @@ bool zyppng::GlibSpawnEngine::start( const char * const *argv, int stdin_fd, int
 
   const char * chdirTo = nullptr;
 
-  if ( !_chroot.empty() ) {
-    const auto chroot = _chroot.c_str();
-    if ( chroot[0] == '\0' )
-    {
-      _chroot = zypp::Pathname();	// ignore empty _chroot
-    }
-    else if ( chroot[0] == '/' && chroot[1] == '\0' )
-    {
-      // If _chroot is '/' do not chroot, but chdir to '/'
-      // unless arglist defines another dir.
-      chdirTo = "/";
-      _chroot = zypp::Pathname();
-    }
+  if ( _chroot == "/" ) {
+    // If _chroot is '/' do not chroot, but chdir to '/'
+    // unless arglist defines another dir.
+    chdirTo = "/";
+    _chroot = zypp::Pathname();
   }
 
   if ( !_workingDirectory.empty() )
