@@ -14,7 +14,6 @@
 #include <zypp/base/Logger.h>
 #include <zypp/base/Regex.h>
 #include <zypp/base/Iterator.h>
-#include <zypp/ui/Selectable.h>
 #include <zypp/PurgeKernels.h>
 #include <zypp/PoolQuery.h>
 #include <zypp/ResPool.h>
@@ -36,7 +35,7 @@
 namespace zypp {
 
   using Flavour                = std::string;
-  using SolvableList           = std::list<sat::Solvable::IdType>;
+  using SolvableList           = std::list<sat::Solvable>;
   using EditionToSolvableMap   = std::map<Edition, SolvableList >;
   using ArchToEditionMap       = std::map<Arch, EditionToSolvableMap >;
 
@@ -69,7 +68,10 @@ namespace zypp {
 
         _detectedRunning = true;
 
-        MIL << "Detected running kernel: " << _runningKernelEdition << " " << _runningKernelFlavour << " " << _kernelArch << std::endl;
+        MIL << "Detected running kernel: Flavour: " << _runningKernelFlavour << " Arch: " << _kernelArch << "\n";
+        for ( const auto &edVar : _runningKernelEditionVariants )
+          MIL << "Edition variant: " << edVar << "\n";
+        MIL << std::endl;
 
       } else {
         MIL << "Failed to detect running kernel: " << errno << std::endl;
@@ -89,29 +91,44 @@ namespace zypp {
 
       version = str::regex_substitute( version, str::regex( "-[^-]*$", str::regex::match_extended | str::regex::newline ), "", true );
 
-      // from purge-kernels script, was copied from kernel-source/rpm/mkspec
-      version = str::regex_substitute( version, str::regex( "\\.0-rc", str::regex::match_extended ), ".rc", true );
-      version = str::regex_substitute( version, str::regex( "-rc\\d+", str::regex::match_extended ), "", true );
-      version = str::regex_substitute( version, str::regex( "-", str::regex::match_extended ), ".", true );
 
-      _runningKernelEdition = Edition( version, release );
+      auto makeRcVariant = [ &release ]( std::string myVersion, const std::string &replace ){
+        // from purge-kernels script, was copied from kernel-source/rpm/mkspec
+        myVersion = str::regex_substitute( myVersion, str::regex( "\\.0-rc", str::regex::match_extended ), replace, true );
+        myVersion = str::regex_substitute( myVersion, str::regex( "-rc\\d+", str::regex::match_extended ), "", true );
+        myVersion = str::regex_substitute( myVersion, str::regex( "-", str::regex::match_extended ), ".", true );
+        return Edition( myVersion, release );
+      };
+
+      _runningKernelEditionVariants.clear();
+      _runningKernelEditionVariants.insert( makeRcVariant( version, "~rc") );
+      _runningKernelEditionVariants.insert( makeRcVariant( version, ".rc") );
+
       _runningKernelFlavour = flavour;
 
       MIL << "Parsed info from uname: " << std::endl;
       MIL << "Kernel Flavour: " << _runningKernelFlavour << std::endl;
-      MIL << "Kernel Edition: " << _runningKernelEdition << std::endl;
+      MIL << "Kernel Edition Variants: \n";
+      for ( const auto &var : _runningKernelEditionVariants )
+        MIL << "    " << var << "\n";
+      MIL << std::endl;
     }
 
-    bool removePackageAndCheck( const sat::Solvable::IdType id, const std::set<sat::Solvable::IdType> &keepList , const std::set<sat::Solvable::IdType> &removeList ) const;
+    bool removePackageAndCheck( const sat::Solvable slv, const std::set<sat::Solvable> &keepList , const std::set<sat::Solvable> &removeList ) const;
     static bool versionMatch ( const Edition &a, const Edition &b );
     void parseKeepSpec();
-    void fillKeepList(const GroupMap &installedKernels, std::set<sat::Solvable::IdType> &keepList , std::set<sat::Solvable::IdType> &removeList ) const;
+    void fillKeepList(const GroupMap &installedKernels, std::set<sat::Solvable> &keepList , std::set<sat::Solvable> &removeList ) const;
 
     std::set<size_t>  _keepLatestOffsets = { 0 };
     std::set<size_t>  _keepOldestOffsets;
     std::set<Edition> _keepSpecificEditions;
     std::string       _uname_r;
-    Edition           _runningKernelEdition;
+    /*!
+     *  A kernel uname can expand into multiple variants especially with rc kernels,
+     *  older rc releases replaced the upstream tag -rc with .rc, however newer code
+     *  replaces it with ~rc. We need to check against all variants to match the correct one.
+     */
+    std::set<Edition> _runningKernelEditionVariants;
     Flavour           _runningKernelFlavour;
     Arch              _kernelArch;
     std::string       _keepSpec = ZConfig::instance().multiversionKernels();
@@ -123,11 +140,11 @@ namespace zypp {
    * tries to remove a the \ref PoolItem \a pi from the pool, solves and checks if no unexpected packages are removed due to the \a validRemovals regex.
    * If the constraint fails the changes are reverted and \a false is returned.
    */
-  bool PurgeKernels::Impl::removePackageAndCheck( const sat::Solvable::IdType id, const std::set<sat::Solvable::IdType> &keepList , const std::set<sat::Solvable::IdType> &removeList ) const
+  bool PurgeKernels::Impl::removePackageAndCheck( const sat::Solvable slv, const std::set<sat::Solvable> &keepList , const std::set<sat::Solvable> &removeList ) const
   {
     const filter::ByStatus toBeUninstalledFilter( &ResStatus::isToBeUninstalled );
 
-    PoolItem pi ( (sat::Solvable(id)) );
+    PoolItem pi ( slv );
 
     auto pool = ResPool::instance();
 
@@ -140,17 +157,17 @@ namespace zypp {
     MIL << "Request to remove package: " << pi << std::endl;
 
     //list of packages that are allowed to be removed automatically.
-    const str::regex validRemovals("(kernel-syms(-.*)?|kgraft-patch(-.*)?|kernel-livepatch(-.*)?|.*-kmp(-.*)?)");
+    const str::regex validRemovals("(kernel-syms(-.*)?|kgraft-patch(-.*)?|kernel-(.*)-livepatch(-.*)?|kernel-livepatch(-.*)?|.*-kmp(-.*)?)");
 
-    if ( ui::asSelectable()( pi )->hasLocks() ) {
+    if ( pi.status().isLocked() ) {
       MIL << "Package " << pi << " is locked by the user, not removing." << std::endl;
       return false;
     }
 
     //remember which packages are already marked for removal, we do not need to check them again
-    std::set< sat::Solvable::IdType> currentSetOfRemovals;
-    for ( auto it = pool.byStatusBegin( toBeUninstalledFilter ); it != pool.byStatusEnd( toBeUninstalledFilter );  it++  ) {
-      currentSetOfRemovals.insert( it->id() );
+    std::set<sat::Solvable> currentSetOfRemovals;
+    for ( const PoolItem & p : pool.byStatus( toBeUninstalledFilter ) ) {
+      currentSetOfRemovals.insert( p.satSolvable() );
     }
 
     pi.status().setToBeUninstalled( ResStatus::USER );
@@ -163,37 +180,61 @@ namespace zypp {
       return false;
     }
 
-    std::set<sat::Solvable::IdType> removedInThisRun;
-    removedInThisRun.insert( pi.id() );
+    std::set<sat::Solvable> removedInThisRun;
+    removedInThisRun.insert( slv );
 
-    for ( auto it = pool.byStatusBegin( toBeUninstalledFilter ); it != pool.byStatusEnd( toBeUninstalledFilter );  it++  ) {
+    for ( const PoolItem & p : pool.byStatus( toBeUninstalledFilter ) ) {
 
       //check if that package is removeable
-      if ( it->status().isByUser()      //this was set by us, ignore it
-           || (currentSetOfRemovals.find( it->id() ) != currentSetOfRemovals.end()) //this was marked by a previous removal, ignore them
+      if ( p.status().isByUser()      //this was set by us, ignore it
+           || (currentSetOfRemovals.find( p.satSolvable() ) != currentSetOfRemovals.end()) //this was marked by a previous removal, ignore them
         )
         continue;
 
       // remember for later we need remove the debugsource and debuginfo packages as well
-      removedInThisRun.insert( it->id() );
+      removedInThisRun.insert( p.satSolvable() );
 
-      MIL << "Package " << PoolItem(*it) << " was marked by the solver for removal." << std::endl;
+      MIL << "Package " << p << " was marked by the solver for removal." << std::endl;
 
       // if we do not plan to remove that package anyway, we need to check if its allowed to be removed ( package in removelist can never be in keep list )
-      if ( removeList.find( it->id() ) != removeList.end() )
+      if ( removeList.find( p.satSolvable() ) != removeList.end() )
         continue;
 
-      if ( keepList.find( it->id() ) != keepList.end() ) {
-        MIL << "Package " << PoolItem(*it) << " is in keep spec, skipping" << pi << std::endl;
+      if ( keepList.find( p.satSolvable() ) != keepList.end() ) {
+        MIL << "Package " << p << " is in keep spec, skipping" << pi << std::endl;
         pi.statusReset();
         return false;
       }
 
-      str::smatch what;
-      if ( !str::regex_match( it->name(), what, validRemovals) ) {
-        MIL << "Package " << PoolItem(*it) << " should not be removed, skipping " << pi << std::endl;
-        pi.statusReset();
-        return false;
+      /*
+       * bsc#1185325 We can not solely rely on name matching to figure out
+       * which packages are kmod's, in SLES from Leap 15.3 forward we have the
+       * kernel-flavour-extra packages ( and others similarly named ) that are basically
+       * a collection of kmod's. So checking the name for .*-kmp(-.*)? is not enough.
+       * We first check if the package provides kmod(*) or ksym(*) and only fall back to name
+       * checking if that is not the case.
+       * Just to be safe I'll leave the regex in the fallback case as well, but it should be completely
+       * redundant now.
+       */
+      bool mostLikelyKmod = false;
+      StrMatcher matchMod( "kmod(*)", Match::GLOB );
+      StrMatcher matchSym( "ksym(*)", Match::GLOB );
+      for ( const auto &prov : p.provides() ) {
+        if ( matchMod.doMatch( prov.detail().name().c_str()) || matchSym.doMatch( prov.detail().name().c_str() ) ) {
+          mostLikelyKmod = true;
+          break;
+        }
+      }
+
+      if ( mostLikelyKmod  ) {
+        MIL << "Package " << p << " is most likely a kmod " << std::endl;
+      } else {
+        str::smatch what;
+        if ( !str::regex_match( p.name(), what, validRemovals) ) {
+          MIL << "Package " << p << " should not be removed, skipping " << pi << std::endl;
+          pi.statusReset();
+          return false;
+        }
       }
     }
 
@@ -201,28 +242,27 @@ namespace zypp {
 
     //now check and mark the -debugsource and -debuginfo packages for this package and all the packages that were removed. Maybe collect it before and just remove here
     MIL << "Trying to remove debuginfo for: " << pi <<"."<<std::endl;
-    for ( const auto id : removedInThisRun ) {
+    for ( sat::Solvable solvable : removedInThisRun ) {
 
-      const auto solvable = sat::Solvable(id);
       if ( solvable.arch() == Arch_noarch ||
            solvable.arch() == Arch_empty )
         continue;
 
-      for ( const auto suffix : { "-debugsource", "-debuginfo" } ) {
+      for ( const char * suffix : { "-debugsource", "-debuginfo" } ) {
         PoolQuery q;
         q.addKind( zypp::ResKind::package );
         q.addDependency( sat::SolvAttr::provides, Capability( solvable.name()+suffix, Rel::EQ, solvable.edition() ) );
         q.setInstalledOnly();
         q.setMatchExact();
 
-        for ( const auto debugPackage : q ) {
+        for ( sat::Solvable debugPackage : q ) {
 
           if ( debugPackage.arch() != solvable.arch() )
             continue;
 
           MIL << "Found debug package for " << solvable << " : " << debugPackage << std::endl;
           //if removing the package fails it will not stop us from going on , so no need to check
-          removePackageAndCheck( debugPackage.id(), keepList, removeList );
+          removePackageAndCheck( debugPackage, keepList, removeList );
         }
       }
     }
@@ -236,12 +276,21 @@ namespace zypp {
    */
   bool PurgeKernels::Impl::versionMatch( const Edition &a, const Edition &b )
   {
+    if ( a == b )
+      return true;
+
     // the build counter should not be considered here, so if there is one we cut it off
-    const auto dotOffset = b.release().find_last_of(".");
-    if ( dotOffset != std::string::npos ) {
-      return a == zypp::Edition( b.version(), b.release().substr( 0, dotOffset ), b.epoch() );
+    const str::regex buildCntRegex( "\\.[0-9]+($|\\.g[0-9a-f]{7}$)", str::regex::match_extended );
+
+    std::string versionStr = b.asString();
+    str::smatch matches;
+    if ( buildCntRegex.matches( versionStr.data(), matches ) ) {
+      if ( matches.size() >= 2 ) {
+        versionStr.replace( matches.begin(0), (matches.end(0) - matches.begin(0))+1, matches[1] );
+        return a == Edition(versionStr);
+      }
     }
-    return a == b;
+    return false;
   }
 
   /*!
@@ -275,7 +324,18 @@ namespace zypp {
       } else {
         str::smatch what;
         if ( !str::regex_match( word, what, specRegex ) ) {
-          _keepSpecificEditions.insert( Edition(word) );
+          // Allow uname_r version numbers by cutting off any postfix that comes after a second dash, including the second dash.
+          // Github-Issue openSUSE/zypper#418
+          std::string_view edition(word);
+          const auto firstDash = word.find_first_of ('-');
+          if ( firstDash != std::string::npos ) {
+            const auto secondDash = word.find_first_of ('-', firstDash+1 );
+            if ( secondDash != std::string::npos ) {
+              WAR << "Ignoring possible flavor postfix:'"<< word.substr (secondDash) <<"' in keep spec: " << word << std::endl;
+              edition = std::string_view( word.c_str (), secondDash );
+            }
+          }
+          _keepSpecificEditions.insert( Edition(IdString(edition)) );
           continue;
         }
 
@@ -316,18 +376,19 @@ namespace zypp {
    * doable. This is also what the perl script did.
    *
    */
-  void PurgeKernels::Impl::fillKeepList( const GroupMap &installedKernels, std::set<sat::Solvable::IdType> &keepList, std::set<sat::Solvable::IdType> &removeList ) const
+  void PurgeKernels::Impl::fillKeepList( const GroupMap &installedKernels, std::set<sat::Solvable> &keepList, std::set<sat::Solvable> &removeList ) const
   {
 
-    const auto markAsKeep = [ &keepList, &removeList ]( const auto &pck ) {
-      MIL << "Marking package " << sat::Solvable(pck) << " as to keep." << std::endl;
+    const auto markAsKeep = [ &keepList, &removeList ]( sat::Solvable pck ) {
+      MIL << "Marking package " << pck << " as to keep." << std::endl;
       keepList.insert( pck ) ;
       removeList.erase( pck );
     };
 
-    const auto versionPredicate = []( const auto &edition ){
-      return [ &edition ]( const auto &elem ) {
-        return versionMatch( edition, elem.first );
+    const auto versionPredicate = []( const auto &editionVariants ){
+      return [ &editionVariants ]( const auto &elem ) {
+        const auto &f = std::bind( versionMatch, std::placeholders::_1, elem.first );
+        return std::any_of( editionVariants.begin(), editionVariants.end(), f );
       };
     };
 
@@ -349,18 +410,25 @@ namespace zypp {
              && ( ( archMap.first == _kernelArch && groupInfo.second.groupFlavour == _runningKernelFlavour )
                   || groupInfo.second.groupType == GroupInfo::Sources ) ) {
 
-          MIL << "Matching packages against running kernel "<< _runningKernelEdition << "-" << _runningKernelFlavour << "-" <<_kernelArch << std::endl;
+          MIL << "Matching packages against running kernel "<< _runningKernelFlavour << "-" <<_kernelArch << "\nVariants:\n";
+          for ( const auto &var : _runningKernelEditionVariants )
+            MIL << var << "\n";
+          MIL << std::endl;
 
-          auto it = std::find_if( map.begin(), map.end(), versionPredicate( _runningKernelEdition ) );
+          const auto &editionPredicate = versionPredicate( _runningKernelEditionVariants );
+          auto it = std::find_if( map.begin(), map.end(), editionPredicate );
           if ( it == map.end() ) {
 
             // If we look at Sources we cannot match the flavour but we still want to keep on checking the rest of the keep spec
             if ( groupInfo.second.groupType != GroupInfo::Sources  ) {
-              MIL << "Running kernel "<< _runningKernelEdition << "-" << _runningKernelFlavour << "-" <<_kernelArch << " not installed."<<std::endl;
+              MIL << "Running kernel " << _runningKernelFlavour << "-" <<_kernelArch << "\n";
+              for ( const auto &var : _runningKernelEditionVariants )
+                MIL << " Possible Variant:" << var << "\n";
+              MIL << "Not installed! \n";
               MIL << "NOT removing any packages for flavor "<<_runningKernelFlavour<<"-"<<_kernelArch<<" ."<<std::endl;
 
               for ( const auto &kernelMap : map ) {
-                for( const auto &pck : kernelMap.second )
+                for( sat::Solvable pck : kernelMap.second )
                   markAsKeep(pck);
               }
               continue;
@@ -370,7 +438,7 @@ namespace zypp {
             // there could be multiple matches here because of rebuild counter, lets try to find the last one
             MIL << "Found possible running candidate edition: " << it->first << std::endl;
             auto nit = it;
-            for ( nit++ ; nit != map.end() && versionMatch( _runningKernelEdition, nit->first ) ; nit++ ) {
+            for ( nit++ ; nit != map.end() && editionPredicate( *nit ) ; nit++ ) {
               MIL << "Found possible more recent running candidate edition: " << nit->first << std::endl;
               it = nit;
             }
@@ -378,7 +446,7 @@ namespace zypp {
 
           // mark all packages of the running version as keep
           if ( it != map.end() ) {
-            for( const auto &pck : it->second ) {
+            for( sat::Solvable pck : it->second ) {
               markAsKeep(pck);
             }
           }
@@ -386,19 +454,23 @@ namespace zypp {
 
         for ( const auto &kernelMap : map ) {
           //if we find one of the running offsets in the keepspec, we add the kernel id the the list of packages to keep
-          if (  _keepOldestOffsets.find( currOff ) != _keepOldestOffsets.end()
-               || _keepLatestOffsets.find( currROff ) != _keepLatestOffsets.end()
-               // a kernel might be explicitely locked by version
-               // this will currently keep all editions that match, so if keep spec has 1-1 , this will keep 1-1 but also all 1-1.n
-               ||  std::find_if( _keepSpecificEditions.begin(), _keepSpecificEditions.end(),
-                    [ edition = &kernelMap.first ]( const auto &elem ) { return versionMatch( *edition, elem ); } ) != _keepSpecificEditions.end() ) {
-
-            for( const auto &pck : kernelMap.second ) {
-              markAsKeep(pck);
-            }
+          if (  _keepOldestOffsets.find( currOff ) != _keepOldestOffsets.end() || _keepLatestOffsets.find( currROff ) != _keepLatestOffsets.end() ) {
+            std::for_each( kernelMap.second.begin(), kernelMap.second.end(), markAsKeep );
           }
           currOff++;
           currROff--;
+
+          // a kernel package might be explicitly locked by version
+          // We need to go over all package name provides ( provides is named like the package ) and match
+          // them against the specified version to know which ones to keep. (bsc#1176740  bsc#1176192)
+          std::for_each( kernelMap.second.begin(), kernelMap.second.end(), [ & ]( sat::Solvable solv ){
+            for ( Capability prov : solv.provides() ) {
+              if ( prov.detail().name() == solv.name() && _keepSpecificEditions.count( prov.detail().ed() ) ) {
+                markAsKeep( solv );
+                break;
+              }
+            }
+          });
         }
       }
     }
@@ -440,9 +512,9 @@ namespace zypp {
 
 
     // packages that we plan to remove
-    std::set<sat::Solvable::IdType> packagesToRemove;
+    std::set<sat::Solvable> packagesToRemove;
 
-    const auto addPackageToMap = [&installedKrnlPackages, &packagesToRemove] ( const GroupInfo::GroupType type, const std::string &ident, const std::string &flavour, const auto &installedKrnlPck ) {
+    const auto addPackageToMap = [&installedKrnlPackages, &packagesToRemove] ( const GroupInfo::GroupType type, const std::string &ident, const std::string &flavour, const sat::Solvable &installedKrnlPck ) {
 
       if ( !installedKrnlPackages.count( ident ) )
         installedKrnlPackages.insert( std::make_pair( ident, GroupInfo(type, flavour) ) );
@@ -460,18 +532,40 @@ namespace zypp {
 
       auto &editionToSolvableMap = groupInfo.archToEdMap[ currArch ];
 
-      const auto currEd  = installedKrnlPck.edition();
-      if ( !editionToSolvableMap.count( currEd ) )
-        editionToSolvableMap.insert( std::make_pair( currEd, SolvableList{} ) );
+      // calculate the "shortest" or most generic edition of all the package name provides
+      // ( the key of the provides is the package name ). This generic edition is used to
+      // group the packages together. This should get us around the issue that uname -r does
+      // not represent the actual rpm package version anymore. ( bsc#1176740 )
+      auto currCount = INT_MAX;
+      Edition edToUse;
+      for ( Capability prov : installedKrnlPck.provides() ) {
+        if ( prov.detail().name() == installedKrnlPck.name() ) {
+          if ( edToUse == Edition::noedition ) {
+            edToUse = installedKrnlPck.edition();
+            const auto &relStr = edToUse.release();
+            currCount = std::count( relStr.begin(), relStr.end(), '.');
+          } else {
+            const auto &pckEd = prov.detail().ed();
+            const auto &relStr = pckEd.release();
+            if ( const auto pntCnt = std::count( relStr.begin(), relStr.end(), '.'); pntCnt < currCount ) {
+              currCount = pntCnt;
+              edToUse = pckEd;
+            }
+          }
+        }
+      }
 
-      editionToSolvableMap[currEd].push_back( installedKrnlPck.id() );
+      if ( !editionToSolvableMap.count( edToUse ) )
+        editionToSolvableMap.insert( std::make_pair( edToUse, SolvableList{} ) );
 
-      //in the first step we collect all packages in this list, then later we will remove the packages we want to explicitely keep
-      packagesToRemove.insert( installedKrnlPck.id() );
+      editionToSolvableMap[edToUse].push_back( installedKrnlPck );
+
+      //in the first step we collect all packages in this list, then later we will remove the packages we want to explicitly keep
+      packagesToRemove.insert( installedKrnlPck );
     };
 
-    // the set of package IDs that have to be kept always
-    std::set<sat::Solvable::IdType> packagesToKeep;
+    // the set of satSolvables that have to be kept always
+    std::set<sat::Solvable> packagesToKeep;
 
     //collect the list of installed kernel packages
     PoolQuery q;
@@ -482,7 +576,7 @@ namespace zypp {
 
     MIL << "Searching for obsolete multiversion kernel packages." << std::endl;
 
-    for ( auto installedKrnlPck : q ) {
+    for ( sat::Solvable installedKrnlPck : q ) {
 
       MIL << "Found installed multiversion kernel package " << installedKrnlPck << std::endl;
 
@@ -510,34 +604,38 @@ namespace zypp {
 
       } else {
 
-        const str::regex explicitelyHandled("kernel-syms(-.*)?|kernel(-.*)?-devel");
+        // if adapting the groups do not forget to explicitly handle the group when querying the matches
+        const str::regex explicitlyHandled("kernel-syms(-.*)?|kernel(-.*)?-devel");
 
         MIL << "Not a kernel package, inspecting more closely " << std::endl;
 
         // we directly handle all noarch packages that export multiversion(kernel)
         if ( installedKrnlPck.arch() == Arch_noarch ) {
 
-          MIL << "Handling package explicitely due to architecture (noarch)."<< std::endl;
+          MIL << "Handling package explicitly due to architecture (noarch)."<< std::endl;
           addPackageToMap( GroupInfo::Sources, installedKrnlPck.name(), "", installedKrnlPck );
 
-        } else if ( str::smatch match; str::regex_match( installedKrnlPck.name(), match, explicitelyHandled ) ) {
+        } else if ( str::smatch match; str::regex_match( installedKrnlPck.name(), match, explicitlyHandled ) ) {
 
           // try to get the flavour from the name
           // if we have a kernel-syms getting no flavour means we have the "default" one, otherwise we use the flavour
           // getting no flavour for a kernel(-*)?-devel means we have the kernel-devel package otherwise the flavour specific one
           // ...yes this is horrible
           std::string flav;
-          if ( match.size() > 1 )  {
+
+          // first group match is a kernel-syms
+          if ( match.size() > 1 && match[1].size() )
+            flav = match[1].substr(1);
+          // second group match is a kernel-flavour-devel
+          else if ( match.size() > 2 &&  match[2].size() )
             flav = match[2].substr(1);
-          } else if ( installedKrnlPck.name() == "kernel-syms" ) {
+          else if ( installedKrnlPck.name() == "kernel-syms" )
             flav = "default";
-          }
 
-          MIL << "Handling package explicitely due to name match."<< std::endl;
+          MIL << "Handling package explicitly due to name match."<< std::endl;
           addPackageToMap ( GroupInfo::RelatedBinaries, installedKrnlPck.name(), flav, installedKrnlPck );
-
         } else {
-          MIL << "Package not explicitely handled" << std::endl;
+          MIL << "Package not explicitly handled" << std::endl;
         }
       }
 
@@ -561,8 +659,8 @@ namespace zypp {
 
     _pimpl->fillKeepList( installedKrnlPackages, packagesToKeep, packagesToRemove );
 
-    for ( const auto id : packagesToRemove )
-      _pimpl->removePackageAndCheck( id, packagesToKeep, packagesToRemove );
+    for ( sat::Solvable slv : packagesToRemove )
+      _pimpl->removePackageAndCheck( slv, packagesToKeep, packagesToRemove );
   }
 
   void PurgeKernels::setUnameR( const std::string &val )
