@@ -21,6 +21,7 @@ extern "C"
 }
 #include <iostream>
 #include <fstream>
+#include <optional>
 #include <zypp/base/LogTools.h>
 #include <zypp/base/IOStream.h>
 #include <zypp-core/base/InputStream>
@@ -315,9 +316,85 @@ namespace zypp
   {
     typedef std::set<std::string> MultiversionSpec;
 
+    /// Settings that follow a changed Target
+    struct TargetDefaults
+    {
+      TargetDefaults()
+      : solver_focus                        ( ResolverFocus::Default )
+      , solver_onlyRequires                 ( false )
+      , solver_allowVendorChange            ( false )
+      , solver_dupAllowDowngrade            ( true )
+      , solver_dupAllowNameChange           ( true )
+      , solver_dupAllowArchChange           ( true )
+      , solver_dupAllowVendorChange         ( true )
+      , solver_cleandepsOnRemove            ( false )
+      , solver_upgradeTestcasesToKeep       ( 2 )
+      , solverUpgradeRemoveDroppedPackages  ( true )
+      {}
+
+      bool consume( const std::string & entry, const std::string & value )
+      {
+        if ( entry == "solver.focus" )
+        {
+          fromString( value, solver_focus );
+        }
+        else if ( entry == "solver.onlyRequires" )
+        {
+          solver_onlyRequires.set( str::strToBool( value, solver_onlyRequires ) );
+        }
+        else if ( entry == "solver.allowVendorChange" )
+        {
+          solver_allowVendorChange.set( str::strToBool( value, solver_allowVendorChange ) );
+        }
+        else if ( entry == "solver.dupAllowDowngrade" )
+        {
+          solver_dupAllowDowngrade.set( str::strToBool( value, solver_dupAllowDowngrade ) );
+        }
+        else if ( entry == "solver.dupAllowNameChange" )
+        {
+          solver_dupAllowNameChange.set( str::strToBool( value, solver_dupAllowNameChange ) );
+        }
+        else if ( entry == "solver.dupAllowArchChange" )
+        {
+          solver_dupAllowArchChange.set( str::strToBool( value, solver_dupAllowArchChange ) );
+        }
+        else if ( entry == "solver.dupAllowVendorChange" )
+        {
+          solver_dupAllowVendorChange.set( str::strToBool( value, solver_dupAllowVendorChange ) );
+        }
+        else if ( entry == "solver.cleandepsOnRemove" )
+        {
+          solver_cleandepsOnRemove.set( str::strToBool( value, solver_cleandepsOnRemove ) );
+        }
+        else if ( entry == "solver.upgradeTestcasesToKeep" )
+        {
+          solver_upgradeTestcasesToKeep.set( str::strtonum<unsigned>( value ) );
+        }
+        else if ( entry == "solver.upgradeRemoveDroppedPackages" )
+        {
+          solverUpgradeRemoveDroppedPackages.restoreToDefault( str::strToBool( value, solverUpgradeRemoveDroppedPackages.getDefault() ) );
+        }
+        else
+          return false;
+
+        return true;
+      }
+
+      ResolverFocus       solver_focus;
+      Option<bool>        solver_onlyRequires;
+      Option<bool>        solver_allowVendorChange;
+      Option<bool>        solver_dupAllowDowngrade;
+      Option<bool>        solver_dupAllowNameChange;
+      Option<bool>        solver_dupAllowArchChange;
+      Option<bool>        solver_dupAllowVendorChange;
+      Option<bool>        solver_cleandepsOnRemove;
+      Option<unsigned>    solver_upgradeTestcasesToKeep;
+      DefaultOption<bool> solverUpgradeRemoveDroppedPackages;
+    };
+
     public:
-      Impl( const Pathname & override_r = Pathname() )
-        : _parsedZyppConf         	( override_r )
+      Impl()
+        : _parsedZyppConf         	( _autodetectZyppConfPath() )
         , cfg_arch                	( defaultSystemArchitecture() )
         , cfg_textLocale          	( defaultTextLocale() )
         , cfg_cache_path		{ "/var/cache/zypp" }
@@ -336,36 +413,10 @@ namespace zypp
         , gpgCheck			( true )
         , repoGpgCheck			( indeterminate )
         , pkgGpgCheck			( indeterminate )
-        , solver_focus			( ResolverFocus::Default )
-        , solver_onlyRequires		( false )
-        , solver_allowVendorChange	( false )
-        , solver_dupAllowDowngrade	( true )
-        , solver_dupAllowNameChange	( true )
-        , solver_dupAllowArchChange	( true )
-        , solver_dupAllowVendorChange	( true )
-        , solver_cleandepsOnRemove	( false )
-        , solver_upgradeTestcasesToKeep	( 2 )
-        , solverUpgradeRemoveDroppedPackages( true )
         , apply_locks_file		( true )
         , pluginsPath			( "/usr/lib/zypp/plugins" )
       {
         MIL << "libzypp: " LIBZYPP_VERSION_STRING << endl;
-        // override_r has higest prio
-        // ZYPP_CONF might override /etc/zypp/zypp.conf
-        if ( _parsedZyppConf.empty() )
-        {
-          _parsedZyppConf = _autodetectZyppConfPath();
-        }
-        else
-        {
-          // Inject this into ZConfig. Be shure this is
-          // allocated via new.
-          // ma: override_r might not be needed anymore since the
-          //     Vendor_test is now able to initialize VendorAttr directly.
-          INT << "Reconfigure to " << _parsedZyppConf << endl;
-          ZConfig::instance()._pimpl.reset( this );
-        }
-
         if ( PathInfo(_parsedZyppConf).isExist() )
         {
           parser::IniDict dict( _parsedZyppConf );
@@ -388,6 +439,9 @@ namespace zypp
               //DBG << (*it).first << "=" << (*it).second << endl;
               if ( section == "main" )
               {
+                if ( _initialTargetDefaults.consume( entry, value ) )
+                  continue;
+
                 if ( entry == "arch" )
                 {
                   Arch carch( value );
@@ -492,46 +546,6 @@ namespace zypp
                 {
                   cfg_kernel_keep_spec = value;
                 }
-                else if ( entry == "solver.focus" )
-                {
-                  fromString( value, solver_focus );
-                }
-                else if ( entry == "solver.onlyRequires" )
-                {
-                  solver_onlyRequires.set( str::strToBool( value, solver_onlyRequires ) );
-                }
-                else if ( entry == "solver.allowVendorChange" )
-                {
-                  solver_allowVendorChange.set( str::strToBool( value, solver_allowVendorChange ) );
-                }
-                else if ( entry == "solver.dupAllowDowngrade" )
-                {
-                  solver_dupAllowDowngrade.set( str::strToBool( value, solver_dupAllowDowngrade ) );
-                }
-                else if ( entry == "solver.dupAllowNameChange" )
-                {
-                  solver_dupAllowNameChange.set( str::strToBool( value, solver_dupAllowNameChange ) );
-                }
-                else if ( entry == "solver.dupAllowArchChange" )
-                {
-                  solver_dupAllowArchChange.set( str::strToBool( value, solver_dupAllowArchChange ) );
-                }
-                else if ( entry == "solver.dupAllowVendorChange" )
-                {
-                  solver_dupAllowVendorChange.set( str::strToBool( value, solver_dupAllowVendorChange ) );
-                }
-                else if ( entry == "solver.cleandepsOnRemove" )
-                {
-                  solver_cleandepsOnRemove.set( str::strToBool( value, solver_cleandepsOnRemove ) );
-                }
-                else if ( entry == "solver.upgradeTestcasesToKeep" )
-                {
-                  solver_upgradeTestcasesToKeep.set( str::strtonum<unsigned>( value ) );
-                }
-                else if ( entry == "solver.upgradeRemoveDroppedPackages" )
-                {
-                  solverUpgradeRemoveDroppedPackages.restoreToDefault( str::strToBool( value, solverUpgradeRemoveDroppedPackages.getDefault() ) );
-                }
                 else if ( entry == "solver.checkSystemFile" )
                 {
                   solver_checkSystemFile = Pathname(value);
@@ -591,8 +605,6 @@ namespace zypp
               }
             }
           }
-          //
-
         }
         else
         {
@@ -616,6 +628,30 @@ namespace zypp
       ~Impl()
       {}
 
+      void notifyTargetChanged()
+      {
+        Pathname newRoot { _autodetectSystemRoot() };
+        MIL << "notifyTargetChanged (" << newRoot << ")" << endl;
+
+        if ( newRoot.emptyOrRoot() ) {
+          _currentTargetDefaults.reset(); // to initial settigns from /
+        }
+        else {
+          _currentTargetDefaults = TargetDefaults();
+
+          Pathname newConf { newRoot/_autodetectZyppConfPath() };
+          if ( PathInfo(newConf).isExist() ) {
+            parser::IniDict dict( newConf );
+            for ( const auto & [entry,value] : dict.entries( "main" ) ) {
+              (*_currentTargetDefaults).consume( entry, value );
+            }
+          }
+          else {
+            MIL << _parsedZyppConf << " not found, using defaults." << endl;
+          }
+        }
+      }
+
     public:
     /** Remember any parsed zypp.conf. */
     Pathname _parsedZyppConf;
@@ -660,17 +696,6 @@ namespace zypp
     DefaultOption<TriBool>	repoGpgCheck;
     DefaultOption<TriBool>	pkgGpgCheck;
 
-    ResolverFocus	solver_focus;
-    Option<bool>	solver_onlyRequires;
-    Option<bool>	solver_allowVendorChange;
-    Option<bool>	solver_dupAllowDowngrade;
-    Option<bool>	solver_dupAllowNameChange;
-    Option<bool>	solver_dupAllowArchChange;
-    Option<bool>	solver_dupAllowVendorChange;
-    Option<bool>	solver_cleandepsOnRemove;
-    Option<unsigned>	solver_upgradeTestcasesToKeep;
-    DefaultOption<bool> solverUpgradeRemoveDroppedPackages;
-
     Pathname solver_checkSystemFile;
     Pathname solver_checkSystemFileDir;
 
@@ -690,6 +715,14 @@ namespace zypp
     /* Other config singleton instances */
     MediaConfig &_mediaConf = MediaConfig::instance();
 
+
+  public:
+    const TargetDefaults & targetDefaults() const { return _currentTargetDefaults ? *_currentTargetDefaults : _initialTargetDefaults; }
+    TargetDefaults &       targetDefaults()       { return _currentTargetDefaults ? *_currentTargetDefaults : _initialTargetDefaults; }
+  private:
+    TargetDefaults                _initialTargetDefaults; ///< Initial TargetDefaults from /
+    std::optional<TargetDefaults> _currentTargetDefaults; ///< TargetDefaults while --root
+
   private:
     // HACK for bnc#906096: let pool re-evaluate multiversion spec
     // if target root changes. ZConfig returns data sensitive to
@@ -812,10 +845,12 @@ namespace zypp
   ZConfig::~ZConfig( )
   {}
 
+  void ZConfig::notifyTargetChanged()
+  { return _pimpl->notifyTargetChanged(); }
+
   Pathname ZConfig::systemRoot() const
   { return _autodetectSystemRoot(); }
 
-
   Pathname ZConfig::repoManagerRoot() const
   {
     return ( _pimpl->cfg_repo_mgr_root_path.empty()
@@ -1074,21 +1109,21 @@ namespace zypp
   void ZConfig::resetRepoGpgCheck()			{ _pimpl->repoGpgCheck.restoreToDefault(); }
   void ZConfig::resetPkgGpgCheck()			{ _pimpl->pkgGpgCheck.restoreToDefault(); }
 
-  ResolverFocus ZConfig::solver_focus() const		{ return _pimpl->solver_focus; }
 
-  bool ZConfig::solver_onlyRequires() const
-  { return _pimpl->solver_onlyRequires; }
+  ResolverFocus ZConfig::solver_focus() const           { return _pimpl->targetDefaults().solver_focus; }
+  bool ZConfig::solver_onlyRequires() const             { return _pimpl->targetDefaults().solver_onlyRequires; }
+  bool ZConfig::solver_allowVendorChange() const        { return _pimpl->targetDefaults().solver_allowVendorChange; }
+  bool ZConfig::solver_dupAllowDowngrade() const        { return _pimpl->targetDefaults().solver_dupAllowDowngrade; }
+  bool ZConfig::solver_dupAllowNameChange() const       { return _pimpl->targetDefaults().solver_dupAllowNameChange; }
+  bool ZConfig::solver_dupAllowArchChange() const       { return _pimpl->targetDefaults().solver_dupAllowArchChange; }
+  bool ZConfig::solver_dupAllowVendorChange() const     { return _pimpl->targetDefaults().solver_dupAllowVendorChange; }
+  bool ZConfig::solver_cleandepsOnRemove() const        { return _pimpl->targetDefaults().solver_cleandepsOnRemove; }
+  unsigned ZConfig::solver_upgradeTestcasesToKeep() const { return _pimpl->targetDefaults().solver_upgradeTestcasesToKeep; }
 
-  bool ZConfig::solver_allowVendorChange() const
-  { return _pimpl->solver_allowVendorChange; }
+  bool ZConfig::solverUpgradeRemoveDroppedPackages() const          { return _pimpl->targetDefaults().solverUpgradeRemoveDroppedPackages; }
+  void ZConfig::setSolverUpgradeRemoveDroppedPackages( bool val_r ) { _pimpl->targetDefaults().solverUpgradeRemoveDroppedPackages.set( val_r ); }
+  void ZConfig::resetSolverUpgradeRemoveDroppedPackages()           { _pimpl->targetDefaults().solverUpgradeRemoveDroppedPackages.restoreToDefault(); }
 
-  bool ZConfig::solver_dupAllowDowngrade() const	{ return _pimpl->solver_dupAllowDowngrade; }
-  bool ZConfig::solver_dupAllowNameChange() const	{ return _pimpl->solver_dupAllowNameChange; }
-  bool ZConfig::solver_dupAllowArchChange() const	{ return _pimpl->solver_dupAllowArchChange; }
-  bool ZConfig::solver_dupAllowVendorChange() const	{ return _pimpl->solver_dupAllowVendorChange; }
-
-  bool ZConfig::solver_cleandepsOnRemove() const
-  { return _pimpl->solver_cleandepsOnRemove; }
 
   Pathname ZConfig::solver_checkSystemFile() const
   { return ( _pimpl->solver_checkSystemFile.empty()
@@ -1098,12 +1133,6 @@ namespace zypp
   { return ( _pimpl->solver_checkSystemFileDir.empty()
       ? (configPath()/"systemCheck.d") : _pimpl->solver_checkSystemFileDir ); }
 
-  unsigned ZConfig::solver_upgradeTestcasesToKeep() const
-  { return _pimpl->solver_upgradeTestcasesToKeep; }
-
-  bool ZConfig::solverUpgradeRemoveDroppedPackages() const		{ return _pimpl->solverUpgradeRemoveDroppedPackages; }
-  void ZConfig::setSolverUpgradeRemoveDroppedPackages( bool val_r )	{ _pimpl->solverUpgradeRemoveDroppedPackages.set( val_r ); }
-  void ZConfig::resetSolverUpgradeRemoveDroppedPackages()		{ _pimpl->solverUpgradeRemoveDroppedPackages.restoreToDefault(); }
 
   namespace
   {
