@@ -17,15 +17,15 @@
 #include <zypp/ExternalProgram.h>
 #include <zypp/base/String.h>
 #include <zypp/base/Gettext.h>
-#include <zypp/base/Sysconfig.h>
+#include <zypp-core/parser/Sysconfig>
 #include <zypp/base/Gettext.h>
 
 #include <zypp/media/MediaCurl.h>
-#include <zypp/media/ProxyInfo.h>
-#include <zypp/media/MediaUserAuth.h>
-#include <zypp/media/CredentialManager.h>
-#include <zypp/media/CurlConfig.h>
-#include <zypp/media/CurlHelper.h>
+#include <zypp-curl/ProxyInfo>
+#include <zypp-curl/auth/CurlAuthData>
+#include <zypp-media/auth/CredentialManager>
+#include <zypp-curl/CurlConfig>
+#include <zypp-curl/private/curlhelper_p.h>
 #include <zypp/Target.h>
 #include <zypp/ZYppFactory.h>
 #include <zypp/ZConfig.h>
@@ -40,6 +40,175 @@
 
 using std::endl;
 
+namespace internal {
+  using namespace zypp;
+  struct ProgressData
+  {
+    ProgressData( CURL *_curl, time_t _timeout = 0, const zypp::Url & _url = zypp::Url(),
+      zypp::ByteCount expectedFileSize_r = 0,
+      zypp::callback::SendReport<zypp::media::DownloadProgressReport> *_report = nullptr );
+
+    CURL	*curl;
+    zypp::Url	url;
+    time_t	timeout;
+    bool	reached;
+    bool      fileSizeExceeded;
+    zypp::callback::SendReport<zypp::media::DownloadProgressReport> *report;
+    zypp::ByteCount _expectedFileSize;
+
+    time_t _timeStart	= 0;	///< Start total stats
+    time_t _timeLast	= 0;	///< Start last period(~1sec)
+    time_t _timeRcv	= 0;	///< Start of no-data timeout
+    time_t _timeNow	= 0;	///< Now
+
+    double _dnlTotal	= 0.0;	///< Bytes to download or 0 if unknown
+    double _dnlLast	= 0.0;	///< Bytes downloaded at period start
+    double _dnlNow	= 0.0;	///< Bytes downloaded now
+
+    int    _dnlPercent= 0;	///< Percent completed or 0 if _dnlTotal is unknown
+
+    double _drateTotal= 0.0;	///< Download rate so far
+    double _drateLast	= 0.0;	///< Download rate in last period
+
+    void updateStats( double dltotal = 0.0, double dlnow = 0.0 );
+
+    int reportProgress() const;
+
+
+    // download rate of the last period (cca 1 sec)
+    double                                        drate_period;
+    // bytes downloaded at the start of the last period
+    double                                        dload_period;
+    // seconds from the start of the download
+    long                                          secs;
+    // average download rate
+    double                                        drate_avg;
+    // last time the progress was reported
+    time_t                                        ltime;
+    // bytes downloaded at the moment the progress was last reported
+    double                                        dload;
+    // bytes uploaded at the moment the progress was last reported
+    double                                        uload;
+  };
+
+
+
+  ProgressData::ProgressData(CURL *_curl, time_t _timeout, const Url &_url, ByteCount expectedFileSize_r, zypp::callback::SendReport< zypp::media::DownloadProgressReport> *_report)
+    : curl( _curl )
+    , url( _url )
+    , timeout( _timeout )
+    , reached( false )
+    , fileSizeExceeded ( false )
+    , report( _report )
+    , _expectedFileSize( expectedFileSize_r )
+  {}
+
+  void ProgressData::updateStats(double dltotal, double dlnow)
+  {
+    time_t now = _timeNow = time(0);
+
+    // If called without args (0.0), recompute based on the last values seen
+    if ( dltotal && dltotal != _dnlTotal )
+      _dnlTotal = dltotal;
+
+    if ( dlnow && dlnow != _dnlNow )
+    {
+      _timeRcv = now;
+      _dnlNow = dlnow;
+    }
+    else if ( !_dnlNow && !_dnlTotal )
+    {
+      // Start time counting as soon as first data arrives.
+      // Skip the connection / redirection time at begin.
+      return;
+    }
+
+    // init or reset if time jumps back
+    if ( !_timeStart || _timeStart > now )
+      _timeStart = _timeLast = _timeRcv = now;
+
+    // timeout condition
+    if ( timeout )
+      reached = ( (now - _timeRcv) > timeout );
+
+    // check if the downloaded data is already bigger than what we expected
+    fileSizeExceeded = _expectedFileSize > 0 && _expectedFileSize < static_cast<ByteCount::SizeType>(_dnlNow);
+
+    // percentage:
+    if ( _dnlTotal )
+      _dnlPercent = int(_dnlNow * 100 / _dnlTotal);
+
+    // download rates:
+    _drateTotal = _dnlNow / std::max( int(now - _timeStart), 1 );
+
+    if ( _timeLast < now )
+    {
+      _drateLast = (_dnlNow - _dnlLast) / int(now - _timeLast);
+      // start new period
+      _timeLast  = now;
+      _dnlLast   = _dnlNow;
+    }
+    else if ( _timeStart == _timeLast )
+      _drateLast = _drateTotal;
+  }
+
+  int ProgressData::reportProgress() const
+  {
+    if ( fileSizeExceeded )
+      return 1;
+    if ( reached )
+      return 1;	// no-data timeout
+    if ( report && !(*report)->progress( _dnlPercent, url, _drateTotal, _drateLast ) )
+      return 1;	// user requested abort
+    return 0;
+  }
+
+  const char * anonymousIdHeader()
+  {
+    // we need to add the release and identifier to the
+    // agent string.
+    // The target could be not initialized, and then this information
+    // is guessed.
+    static const std::string _value(
+      str::trim( str::form(
+        "X-ZYpp-AnonymousId: %s",
+        Target::anonymousUniqueId( Pathname()/*guess root*/ ).c_str() ) )
+      );
+    return _value.c_str();
+  }
+
+  const char * distributionFlavorHeader()
+  {
+    // we need to add the release and identifier to the
+    // agent string.
+    // The target could be not initialized, and then this information
+    // is guessed.
+    static const std::string _value(
+      str::trim( str::form(
+        "X-ZYpp-DistributionFlavor: %s",
+        Target::distributionFlavor( Pathname()/*guess root*/ ).c_str() ) )
+      );
+    return _value.c_str();
+  }
+
+  const char * agentString()
+  {
+    // we need to add the release and identifier to the
+    // agent string.
+    // The target could be not initialized, and then this information
+    // is guessed.
+    static const std::string _value(
+      str::form(
+        "ZYpp " LIBZYPP_VERSION_STRING " (curl %s) %s"
+        , curl_version_info(CURLVERSION_NOW)->version
+        , Target::targetDistribution( Pathname()/*guess root*/ ).c_str()
+        )
+      );
+    return _value.c_str();
+  }
+}
+
+
 using namespace internal;
 using namespace zypp::base;
 
@@ -47,121 +216,6 @@ namespace zypp {
 
   namespace media {
 
-  namespace {
-    struct ProgressData
-    {
-      ProgressData( CURL *_curl, time_t _timeout = 0, const Url & _url = Url(),
-                    ByteCount expectedFileSize_r = 0,
-		    callback::SendReport<DownloadProgressReport> *_report = nullptr )
-        : curl( _curl )
-	, url( _url )
-	, timeout( _timeout )
-        , reached( false )
-        , fileSizeExceeded ( false )
-        , report( _report )
-        , _expectedFileSize( expectedFileSize_r )
-      {}
-
-      CURL	*curl;
-      Url	url;
-      time_t	timeout;
-      bool	reached;
-      bool      fileSizeExceeded;
-      callback::SendReport<DownloadProgressReport> *report;
-      ByteCount _expectedFileSize;
-
-      time_t _timeStart	= 0;	///< Start total stats
-      time_t _timeLast	= 0;	///< Start last period(~1sec)
-      time_t _timeRcv	= 0;	///< Start of no-data timeout
-      time_t _timeNow	= 0;	///< Now
-
-      double _dnlTotal	= 0.0;	///< Bytes to download or 0 if unknown
-      double _dnlLast	= 0.0;	///< Bytes downloaded at period start
-      double _dnlNow	= 0.0;	///< Bytes downloaded now
-
-      int    _dnlPercent= 0;	///< Percent completed or 0 if _dnlTotal is unknown
-
-      double _drateTotal= 0.0;	///< Download rate so far
-      double _drateLast	= 0.0;	///< Download rate in last period
-
-      void updateStats( double dltotal = 0.0, double dlnow = 0.0 )
-      {
-	time_t now = _timeNow = time(0);
-
-	// If called without args (0.0), recompute based on the last values seen
-	if ( dltotal && dltotal != _dnlTotal )
-	  _dnlTotal = dltotal;
-
-	if ( dlnow && dlnow != _dnlNow )
-	{
-	  _timeRcv = now;
-	  _dnlNow = dlnow;
-	}
-	else if ( !_dnlNow && !_dnlTotal )
-	{
-	  // Start time counting as soon as first data arrives.
-	  // Skip the connection / redirection time at begin.
-	  return;
-	}
-
-	// init or reset if time jumps back
-	if ( !_timeStart || _timeStart > now )
-	  _timeStart = _timeLast = _timeRcv = now;
-
-	// timeout condition
-	if ( timeout )
-	  reached = ( (now - _timeRcv) > timeout );
-
-        // check if the downloaded data is already bigger than what we expected
-  	fileSizeExceeded = _expectedFileSize > 0 && _expectedFileSize < static_cast<ByteCount::SizeType>(_dnlNow);
-
-	// percentage:
-	if ( _dnlTotal )
-	  _dnlPercent = int(_dnlNow * 100 / _dnlTotal);
-
-	// download rates:
-	_drateTotal = _dnlNow / std::max( int(now - _timeStart), 1 );
-
-	if ( _timeLast < now )
-	{
-	  _drateLast = (_dnlNow - _dnlLast) / int(now - _timeLast);
-	  // start new period
-	  _timeLast  = now;
-	  _dnlLast   = _dnlNow;
-	}
-	else if ( _timeStart == _timeLast )
-	  _drateLast = _drateTotal;
-      }
-
-      int reportProgress() const
-      {
-        if ( fileSizeExceeded )
-          return 1;
-	if ( reached )
-	  return 1;	// no-data timeout
-	if ( report && !(*report)->progress( _dnlPercent, url, _drateTotal, _drateLast ) )
-	  return 1;	// user requested abort
-	return 0;
-      }
-
-
-      // download rate of the last period (cca 1 sec)
-      double                                        drate_period;
-      // bytes downloaded at the start of the last period
-      double                                        dload_period;
-      // seconds from the start of the download
-      long                                          secs;
-      // average download rate
-      double                                        drate_avg;
-      // last time the progress was reported
-      time_t                                        ltime;
-      // bytes downloaded at the moment the progress was last reported
-      double                                        dload;
-      // bytes uploaded at the moment the progress was last reported
-      double                                        uload;
-    };
-  }
-
 Pathname MediaCurl::_cookieFile = "/var/lib/YaST2/cookies";
 
 // we use this define to unbloat code as this C setting option
@@ -180,9 +234,9 @@ Pathname MediaCurl::_cookieFile = "/var/lib/YaST2/cookies";
 
 MediaCurl::MediaCurl( const Url &      url_r,
                       const Pathname & attach_point_hint_r )
-    : MediaHandler( url_r, attach_point_hint_r,
-                    "/", // urlpath at attachpoint
-                    true ), // does_download
+    : MediaNetworkCommonHandler( url_r, attach_point_hint_r,
+                                 "/", // urlpath at attachpoint
+                                 true ), // does_download
       _curl( NULL ),
       _customHeaders(0L)
 {
@@ -219,12 +273,6 @@ Url MediaCurl::clearQueryString(const Url &url) const
   return internal::clearQueryString(url);
 }
 
-TransferSettings & MediaCurl::settings()
-{
-    return _settings;
-}
-
-
 void MediaCurl::setCookieFile( const Pathname &fileName )
 {
   _cookieFile = fileName;
@@ -260,8 +308,7 @@ void MediaCurl::checkProtocol(const Url &url) const
 void MediaCurl::setupEasy()
 {
   {
-    char *ptr = getenv("ZYPP_MEDIA_CURL_DEBUG");
-    _curlDebug = (ptr && *ptr) ? str::strtonum<long>( ptr) : 0L;
+    _curlDebug = env::ZYPP_MEDIA_CURL_DEBUG();
     if( _curlDebug > 0)
     {
       curl_easy_setopt( _curl, CURLOPT_VERBOSE, 1L);
@@ -315,13 +362,10 @@ void MediaCurl::setupEasy()
   }
 
   /** Force IPv4/v6 */
-  if ( env::ZYPP_MEDIA_CURL_IPRESOLVE() )
+  switch ( env::ZYPP_MEDIA_CURL_IPRESOLVE() )
   {
-    switch ( env::ZYPP_MEDIA_CURL_IPRESOLVE() )
-    {
-      case 4: SET_OPTION(CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4); break;
-      case 6: SET_OPTION(CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V6); break;
-    }
+    case 4: SET_OPTION(CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4); break;
+    case 6: SET_OPTION(CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V6); break;
   }
 
  /**
@@ -346,7 +390,10 @@ void MediaCurl::setupEasy()
   {
 #if CURLVERSION_AT_LEAST(7,19,4)
     // restrict following of redirections from https to https only
-    SET_OPTION( CURLOPT_REDIR_PROTOCOLS, CURLPROTO_HTTPS );
+    if ( _url.getHost() == "download.opensuse.org" )
+      SET_OPTION( CURLOPT_REDIR_PROTOCOLS, CURLPROTO_HTTP | CURLPROTO_HTTPS );
+    else
+      SET_OPTION( CURLOPT_REDIR_PROTOCOLS, CURLPROTO_HTTPS );
 #endif
 
     if( _settings.verifyPeerEnabled() ||
@@ -380,6 +427,26 @@ void MediaCurl::setupEasy()
 
   SET_OPTION(CURLOPT_USERAGENT, _settings.userAgentString().c_str() );
 
+  /* Fixes bsc#1174011 "auth=basic ignored in some cases"
+       * We should proactively add the password to the request if basic auth is configured
+       * and a password is available in the credentials but not in the URL.
+       *
+       * We will be a bit paranoid here and require that the URL has a user embedded, otherwise we go the default route
+       * and ask the server first about the auth method
+       */
+  if ( _settings.authType() == "basic"
+       && _settings.username().size()
+       && !_settings.password().size() ) {
+
+    CredentialManager cm(CredManagerOptions(ZConfig::instance().repoManagerRoot()));
+    const auto cred = cm.getCred( _url );
+    if ( cred && cred->valid() ) {
+      if ( !_settings.username().size() )
+        _settings.setUsername(cred->username());
+      _settings.setPassword(cred->password());
+    }
+  }
+
   /*---------------------------------------------------------------*
    CURLOPT_USERPWD: [user name]:[password]
 
@@ -397,7 +464,7 @@ void MediaCurl::setupEasy()
     if( auth != CURLAUTH_NONE)
     {
       DBG << "Enabling HTTP authentication methods: " << use_auth
-	  << " (CURLOPT_HTTPAUTH=" << auth << ")" << std::endl;
+          << " (CURLOPT_HTTPAUTH=" << auth << ")" << std::endl;
       SET_OPTION(CURLOPT_HTTPAUTH, auth);
     }
   }
@@ -421,11 +488,11 @@ void MediaCurl::setupEasy()
       CurlConfig curlconf;
       CurlConfig::parseConfig(curlconf); // parse ~/.curlrc
       if ( curlconf.proxyuserpwd.empty() )
-	DBG << "Proxy: ~/.curlrc does not contain the proxy-user option" << endl;
+        DBG << "Proxy: ~/.curlrc does not contain the proxy-user option" << endl;
       else
       {
-	proxyuserpwd = curlconf.proxyuserpwd;
-	DBG << "Proxy: using proxy-user from ~/.curlrc" << endl;
+        proxyuserpwd = curlconf.proxyuserpwd;
+        DBG << "Proxy: using proxy-user from ~/.curlrc" << endl;
       }
     }
     else
@@ -484,13 +551,11 @@ void MediaCurl::setupEasy()
     SET_OPTION(CURLOPT_PROXY_TRANSFER_MODE, 1L );
 #endif
   // append settings custom headers to curl
-  for ( TransferSettings::Headers::const_iterator it = vol_settings.headersBegin();
-        it != vol_settings.headersEnd();
-        ++it )
+  for ( const auto &header : vol_settings.headers() )
   {
     // MIL << "HEADER " << *it << std::endl;
 
-      _customHeaders = curl_slist_append(_customHeaders, it->c_str());
+      _customHeaders = curl_slist_append(_customHeaders, header.c_str());
       if ( !_customHeaders )
           ZYPP_THROW(MediaCurlInitException(_url));
   }
@@ -581,17 +646,20 @@ Url MediaCurl::getFileUrl( const Pathname & filename_r ) const
 
 ///////////////////////////////////////////////////////////////////
 
-void MediaCurl::getFile(const Pathname & filename , const ByteCount &expectedFileSize_r) const
+void MediaCurl::getFile( const OnMediaLocation &file ) const
 {
     // Use absolute file name to prevent access of files outside of the
     // hierarchy below the attach point.
-    getFileCopy(filename, localPath(filename).absolutename(), expectedFileSize_r);
+    getFileCopy( file, localPath(file.filename()).absolutename() );
 }
 
 ///////////////////////////////////////////////////////////////////
 
-void MediaCurl::getFileCopy( const Pathname & filename , const Pathname & target, const ByteCount &expectedFileSize_r ) const
+void MediaCurl::getFileCopy( const OnMediaLocation & srcFile , const Pathname & target ) const
 {
+
+  const auto &filename = srcFile.filename();
+
   callback::SendReport<DownloadProgressReport> report;
 
   Url fileurl(getFileUrl(filename));
@@ -602,7 +670,7 @@ void MediaCurl::getFileCopy( const Pathname & filename , const Pathname & target
   {
     try
     {
-      doGetFileCopy(filename, target, report, expectedFileSize_r);
+      doGetFileCopy( srcFile, target, report );
       retry = false;
     }
     // retry with proper authentication data
@@ -621,9 +689,9 @@ void MediaCurl::getFileCopy( const Pathname & filename , const Pathname & target
     {
       media::DownloadProgressReport::Error reason = media::DownloadProgressReport::ERROR;
       if( typeid(excpt_r) == typeid( media::MediaFileNotFoundException )  ||
-	  typeid(excpt_r) == typeid( media::MediaNotAFileException ) )
+          typeid(excpt_r) == typeid( media::MediaNotAFileException ) )
       {
-	reason = media::DownloadProgressReport::NOT_FOUND;
+        reason = media::DownloadProgressReport::NOT_FOUND;
       }
       report->finish(fileurl, reason, excpt_r.asUserHistory());
       ZYPP_RETHROW(excpt_r);
@@ -684,14 +752,14 @@ void MediaCurl::evaluateCurlCode(const Pathname &filename,
       switch ( code )
       {
       case CURLE_UNSUPPORTED_PROTOCOL:
-	  err = " Unsupported protocol";
-	  if ( !_lastRedirect.empty() )
-	  {
-	    err += " or redirect (";
-	    err += _lastRedirect;
-	    err += ")";
-	  }
-	  break;
+          err = " Unsupported protocol";
+          if ( !_lastRedirect.empty() )
+          {
+            err += " or redirect (";
+            err += _lastRedirect;
+            err += ")";
+          }
+          break;
       case CURLE_URL_MALFORMAT:
       case CURLE_URL_MALFORMAT_USER:
           err = " Bad URL";
@@ -731,9 +799,9 @@ void MediaCurl::evaluateCurlCode(const Pathname &filename,
           case 403:
           {
             std::string msg403;
-	    if ( url.getHost().find(".suse.com") != std::string::npos )
-	      msg403 = _("Visit the SUSE Customer Center to check whether your registration is valid and has not expired.");
-	    else if (url.asString().find("novell.com") != std::string::npos)
+            if ( url.getHost().find(".suse.com") != std::string::npos )
+              msg403 = _("Visit the SUSE Customer Center to check whether your registration is valid and has not expired.");
+            else if (url.asString().find("novell.com") != std::string::npos)
               msg403 = _("Visit the Novell Customer Center to check whether your registration is valid and has not expired.");
             ZYPP_THROW(MediaForbiddenException(url, msg403));
           }
@@ -777,8 +845,8 @@ void MediaCurl::evaluateCurlCode(const Pathname &filename,
         break;
       case CURLE_PARTIAL_FILE:
       case CURLE_OPERATION_TIMEDOUT:
-	timeout_reached	= true; // fall though to TimeoutException
-	// fall though...
+        timeout_reached	= true; // fall though to TimeoutException
+        // fall though...
       case CURLE_ABORTED_BY_CALLBACK:
          if( timeout_reached )
         {
@@ -968,8 +1036,8 @@ bool MediaCurl::detectDirIndex() const
   bool      not_a_file = false;
   char     *ptr = NULL;
   CURLcode  ret = curl_easy_getinfo( _curl,
-				     CURLINFO_EFFECTIVE_URL,
-				     &ptr);
+                                     CURLINFO_EFFECTIVE_URL,
+                                     &ptr);
   if ( ret == CURLE_OK && ptr != NULL)
   {
     try
@@ -978,11 +1046,11 @@ bool MediaCurl::detectDirIndex() const
       std::string path( eurl.getPathName());
       if( !path.empty() && path != "/" && *path.rbegin() == '/')
       {
-	DBG << "Effective url ("
-	    << eurl
-	    << ") seems to provide the index of a directory"
-	    << endl;
-	not_a_file = true;
+        DBG << "Effective url ("
+            << eurl
+            << ") seems to provide the index of a directory"
+            << endl;
+        not_a_file = true;
       }
     }
     catch( ... )
@@ -994,13 +1062,13 @@ bool MediaCurl::detectDirIndex() const
 
 ///////////////////////////////////////////////////////////////////
 
-void MediaCurl::doGetFileCopy(const Pathname & filename , const Pathname & target, callback::SendReport<DownloadProgressReport> & report, const ByteCount &expectedFileSize_r, RequestOptions options ) const
+void MediaCurl::doGetFileCopy( const OnMediaLocation &srcFile , const Pathname & target, callback::SendReport<DownloadProgressReport> & report, RequestOptions options ) const
 {
     Pathname dest = target.absolutename();
     if( assert_dir( dest.dirname() ) )
     {
       DBG << "assert_dir " << dest.dirname() << " failed" << endl;
-      ZYPP_THROW( MediaSystemException(getFileUrl(filename), "System error on " + dest.dirname().asString()) );
+      ZYPP_THROW( MediaSystemException(getFileUrl(srcFile.filename()), "System error on " + dest.dirname().asString()) );
     }
 
     ManagedFile destNew { target.extend( ".new.zypp.XXXXXX" ) };
@@ -1009,23 +1077,23 @@ void MediaCurl::doGetFileCopy(const Pathname & filename , const Pathname & targe
       AutoFREE<char> buf { ::strdup( (*destNew).c_str() ) };
       if( ! buf )
       {
-	ERR << "out of memory for temp file name" << endl;
-	ZYPP_THROW(MediaSystemException(getFileUrl(filename), "out of memory for temp file name"));
+        ERR << "out of memory for temp file name" << endl;
+        ZYPP_THROW(MediaSystemException(getFileUrl(srcFile.filename()), "out of memory for temp file name"));
       }
 
       AutoFD tmp_fd { ::mkostemp( buf, O_CLOEXEC ) };
       if( tmp_fd == -1 )
       {
-	ERR << "mkstemp failed for file '" << destNew << "'" << endl;
-	ZYPP_THROW(MediaWriteException(destNew));
+        ERR << "mkstemp failed for file '" << destNew << "'" << endl;
+        ZYPP_THROW(MediaWriteException(destNew));
       }
       destNew = ManagedFile( (*buf), filesystem::unlink );
 
       file = ::fdopen( tmp_fd, "we" );
       if ( ! file )
       {
-	ERR << "fopen failed for file '" << destNew << "'" << endl;
-	ZYPP_THROW(MediaWriteException(destNew));
+        ERR << "fopen failed for file '" << destNew << "'" << endl;
+        ZYPP_THROW(MediaWriteException(destNew));
       }
       tmp_fd.resetDispose();	// don't close it here! ::fdopen moved ownership to file
     }
@@ -1046,7 +1114,7 @@ void MediaCurl::doGetFileCopy(const Pathname & filename , const Pathname & targe
     }
     try
     {
-      doGetFileCopyFile(filename, dest, file, report, expectedFileSize_r, options);
+      doGetFileCopyFile( srcFile, dest, file, report, options);
     }
     catch (Exception &e)
     {
@@ -1073,7 +1141,7 @@ void MediaCurl::doGetFileCopy(const Pathname & filename , const Pathname & targe
     }
     else
     {
-      WAR << "Could not get the reponse code." << endl;
+      WAR << "Could not get the response code." << endl;
     }
 
     if (modified || infoRet != CURLE_OK)
@@ -1104,9 +1172,9 @@ void MediaCurl::doGetFileCopy(const Pathname & filename , const Pathname & targe
 
 ///////////////////////////////////////////////////////////////////
 
-void MediaCurl::doGetFileCopyFile(const Pathname & filename , const Pathname & dest, FILE *file, callback::SendReport<DownloadProgressReport> & report, const ByteCount &expectedFileSize_r, RequestOptions options ) const
+void MediaCurl::doGetFileCopyFile( const OnMediaLocation & srcFile, const Pathname & dest, FILE *file, callback::SendReport<DownloadProgressReport> & report, RequestOptions options ) const
 {
-    DBG << filename.asString() << endl;
+    DBG << srcFile.filename().asString() << endl;
 
     if(!_url.isValid())
       ZYPP_THROW(MediaBadUrlException(_url));
@@ -1114,7 +1182,7 @@ void MediaCurl::doGetFileCopyFile(const Pathname & filename , const Pathname & d
     if(_url.getHost().empty())
       ZYPP_THROW(MediaBadUrlEmptyHostException(_url));
 
-    Url url(getFileUrl(filename));
+    Url url(getFileUrl(srcFile.filename()));
 
     DBG << "URL: " << url.asString() << endl;
     // Use URL without options and without username and passwd
@@ -1145,7 +1213,7 @@ void MediaCurl::doGetFileCopyFile(const Pathname & filename , const Pathname & d
     }
 
     // Set callback and perform.
-    ProgressData progressData(_curl, _settings.timeout(), url, expectedFileSize_r, &report);
+    internal::ProgressData progressData(_curl, _settings.timeout(), url, srcFile.downloadSize(), &report);
     if (!(options & OPTION_NO_REPORT_START))
       report->start(url, dest);
     if ( curl_easy_setopt( _curl, CURLOPT_PROGRESSDATA, &progressData ) != 0 ) {
@@ -1163,14 +1231,14 @@ void MediaCurl::doGetFileCopyFile(const Pathname & filename , const Pathname & d
       long httpReturnCode = 33;
       if ( curl_easy_getinfo( _curl, CURLINFO_RESPONSE_CODE, &httpReturnCode ) == CURLE_OK && httpReturnCode == 200 )
       {
-	long conditionUnmet = 33;
-	if ( curl_easy_getinfo( _curl, CURLINFO_CONDITION_UNMET, &conditionUnmet ) == CURLE_OK && conditionUnmet )
-	{
-	  WAR << "TIMECONDITION unmet - retry without." << endl;
-	  curl_easy_setopt(_curl, CURLOPT_TIMECONDITION, CURL_TIMECOND_NONE);
-	  curl_easy_setopt(_curl, CURLOPT_TIMEVALUE, 0L);
-	  ret = curl_easy_perform( _curl );
-	}
+        long conditionUnmet = 33;
+        if ( curl_easy_getinfo( _curl, CURLINFO_CONDITION_UNMET, &conditionUnmet ) == CURLE_OK && conditionUnmet )
+        {
+          WAR << "TIMECONDITION unmet - retry without." << endl;
+          curl_easy_setopt(_curl, CURLOPT_TIMECONDITION, CURL_TIMECOND_NONE);
+          curl_easy_setopt(_curl, CURLOPT_TIMEVALUE, 0L);
+          ret = curl_easy_perform( _curl );
+        }
       }
     }
 #endif
@@ -1193,7 +1261,7 @@ void MediaCurl::doGetFileCopyFile(const Pathname & filename , const Pathname & d
         if ( progressData.fileSizeExceeded )
           ZYPP_THROW(MediaFileSizeExceededException(url, progressData._expectedFileSize));
 
-        evaluateCurlCode( filename, ret, progressData.reached );
+        evaluateCurlCode( srcFile.filename(), ret, progressData.reached );
       }
       catch ( const MediaException &e ) {
         // some error, we are not sure about file existence, rethrw
@@ -1204,7 +1272,7 @@ void MediaCurl::doGetFileCopyFile(const Pathname & filename , const Pathname & d
 #if DETECT_DIR_INDEX
     if (!ret && detectDirIndex())
       {
-	ZYPP_THROW(MediaNotAFileException(_url, filename));
+        ZYPP_THROW(MediaNotAFileException(_url, filename));
       }
 #endif // DETECT_DIR_INDEX
 }
@@ -1223,7 +1291,7 @@ void MediaCurl::getDir( const Pathname & dirname, bool recurse_r ) const
       switch ( it->type ) {
       case filesystem::FT_NOT_AVAIL: // old directory.yast contains no typeinfo at all
       case filesystem::FT_FILE:
-        getFile( filename, 0 );
+        getFile( OnMediaLocation( filename ) );
         break;
       case filesystem::FT_DIR: // newer directory.yast contain at least directory info
         if ( recurse_r ) {
@@ -1262,7 +1330,7 @@ void MediaCurl::getDirInfo( filesystem::DirContent & retlist,
 //
 int MediaCurl::aliveCallback( void *clientp, double /*dltotal*/, double dlnow, double /*ultotal*/, double /*ulnow*/ )
 {
-  ProgressData *pdata = reinterpret_cast<ProgressData *>( clientp );
+  internal::ProgressData *pdata = reinterpret_cast<internal::ProgressData *>( clientp );
   if( pdata )
   {
     // Do not propagate dltotal in alive callbacks. MultiCurl uses this to
@@ -1276,7 +1344,7 @@ int MediaCurl::aliveCallback( void *clientp, double /*dltotal*/, double dlnow, d
 
 int MediaCurl::progressCallback( void *clientp, double dltotal, double dlnow, double ultotal, double ulnow )
 {
-  ProgressData *pdata = reinterpret_cast<ProgressData *>( clientp );
+  internal::ProgressData *pdata = reinterpret_cast<internal::ProgressData *>( clientp );
   if( pdata )
   {
     // work around curl bug that gives us old data
@@ -1292,7 +1360,7 @@ int MediaCurl::progressCallback( void *clientp, double dltotal, double dlnow, do
 
 CURL *MediaCurl::progressCallback_getcurl( void *clientp )
 {
-  ProgressData *pdata = reinterpret_cast<ProgressData *>(clientp);
+  internal::ProgressData *pdata = reinterpret_cast<internal::ProgressData *>(clientp);
   return pdata ? pdata->curl : 0;
 }
 
@@ -1319,7 +1387,7 @@ std::string MediaCurl::getAuthHint() const
  */
 void MediaCurl::resetExpectedFileSize(void *clientp, const ByteCount &expectedFileSize)
 {
-  ProgressData *data = reinterpret_cast<ProgressData *>(clientp);
+  internal::ProgressData *data = reinterpret_cast<internal::ProgressData *>(clientp);
   if ( data ) {
     data->_expectedFileSize = expectedFileSize;
   }
@@ -1431,7 +1499,6 @@ bool MediaCurl::authenticate(const std::string & availAuthTypes, bool firstTry)
 //need a out of line definiton, otherwise vtable is emitted for every translation unit
 MediaCurl::Callbacks::~Callbacks() {}
 
-
   } // namespace media
 } // namespace zypp
 //
