@@ -83,7 +83,11 @@ namespace zypp
 
       BOOST_MPL_ASSERT_RELATION( CapDetail::CAP_AND,       ==, REL_AND );
       BOOST_MPL_ASSERT_RELATION( CapDetail::CAP_OR,        ==, REL_OR );
+      BOOST_MPL_ASSERT_RELATION( CapDetail::CAP_COND,      ==, REL_COND );
+      BOOST_MPL_ASSERT_RELATION( CapDetail::CAP_UNLESS,    ==, REL_UNLESS );
+      BOOST_MPL_ASSERT_RELATION( CapDetail::CAP_ELSE,      ==, REL_ELSE );
       BOOST_MPL_ASSERT_RELATION( CapDetail::CAP_WITH,      ==, REL_WITH );
+      BOOST_MPL_ASSERT_RELATION( CapDetail::CAP_WITHOUT,   ==, REL_WITHOUT );
       BOOST_MPL_ASSERT_RELATION( CapDetail::CAP_NAMESPACE, ==, REL_NAMESPACE );
       BOOST_MPL_ASSERT_RELATION( CapDetail::CAP_ARCH,      ==, REL_ARCH );
 
@@ -110,6 +114,8 @@ namespace zypp
       static void logSat( CPool *, void *data, int type, const char *logString )
       {
         //                            "1234567890123456789012345678901234567890
+        if ( 0 == strncmp( logString, "job: drop orphaned", 18 ) )
+          return;
         if ( 0 == strncmp( logString, "job: user installed", 19 ) )
           return;
         if ( 0 == strncmp( logString, "job: multiversion", 17 ) )
@@ -339,7 +345,7 @@ namespace zypp
         // NOTE: the explicit ::repo_free above asserts all solvables are memset(0)!
         if ( !_pool->urepos )
         {
-          _serialIDs.setDirty();	// Indicate resusePoolIDs - ResPool must also invalidate it's PoolItems
+          _serialIDs.setDirty();	// Indicate resusePoolIDs - ResPool must also invalidate its PoolItems
           ::pool_freeallrepos( _pool, /*resusePoolIDs*/true );
         }
       }
