@@ -1109,9 +1109,9 @@ void RpmDb::getData( const std::string & name_r, const Edition & ed_r,
 ///////////////////////////////////////////////////////////////////
 namespace
 {
-  struct RpmlogCapture : public std::string
+  struct RpmlogCapture : public std::vector<std::string>
   {
-    RpmlogCapture() : _cap(this)
+    RpmlogCapture()
     {
       rpmlogSetCallback( rpmLogCB, this );
       _oldMask = rpmlogSetMask( RPMLOG_UPTO( RPMLOG_PRI(RPMLOG_INFO) ) );
@@ -1128,15 +1128,27 @@ namespace
 
     int rpmLog( rpmlogRec rec_r )
     {
-      if ( _cap ) (*_cap) += rpmlogRecMessage( rec_r );
+      std::string l { ::rpmlogRecMessage( rec_r ) };  // NL terminated line!
+      l.pop_back(); // strip trailing NL
+      push_back( std::move(l) );
       return 0;
     }
 
   private:
-    std::string * _cap;
     int _oldMask = 0;
   };
 
+  std::ostream & operator<<( std::ostream & str, const RpmlogCapture & obj )
+  {
+    char sep = '\0';
+    for ( const auto & l : obj ) {
+      if ( sep ) str << sep; else sep = '\n';
+      str << l;
+    }
+    return str;
+  }
+
+
   RpmDb::CheckPackageResult doCheckPackageSig( const Pathname & path_r,			// rpm file to check
                                                const Pathname & root_r,			// target root
                                                bool  requireGPGSig_r,			// whether no gpg signature is to be reported
@@ -1160,60 +1172,80 @@ namespace
     rpmts ts = ::rpmtsCreate();
     ::rpmtsSetRootDir( ts, root_r.c_str() );
     ::rpmtsSetVSFlags( ts, RPMVSF_DEFAULT );
-
-    rpmQVKArguments_s qva;
-    memset( &qva, 0, sizeof(rpmQVKArguments_s) );
-#ifndef HAVE_RPM_VERIFY_TRANSACTION_STEP
-    // Legacy: In rpm >= 4.15 qva_flags symbols don't exist
-    // and qva_flags is not used in signature checking at all.
-    qva.qva_flags = (VERIFY_DIGEST|VERIFY_SIGNATURE);
-#else
+#ifdef HAVE_RPM_VERIFY_TRANSACTION_STEP
     ::rpmtsSetVfyFlags( ts, RPMVSF_DEFAULT );
 #endif
+
     RpmlogCapture vresult;
     LocaleGuard guard( LC_ALL, "C" );	// bsc#1076415: rpm log output is localized, but we need to parse it :(
+    static rpmQVKArguments_s qva = ([](){ rpmQVKArguments_s qva; memset( &qva, 0, sizeof(rpmQVKArguments_s) ); return qva; })();
     int res = ::rpmVerifySignatures( &qva, ts, fd, path_r.basename().c_str() );
     guard.restore();
 
     ts = rpmtsFree(ts);
     ::Fclose( fd );
 
-    // results per line...
-    //     Header V3 RSA/SHA256 Signature, key ID 3dbdc284: OK
-    //     Header SHA1 digest: OK (a60386347863affefef484ff1f26c889373eb094)
-    //     V3 RSA/SHA256 Signature, key ID 3dbdc284: OK
-    //     MD5 digest: OK (fd5259fe677a406951dcb2e9d08c4dcc)
-    //
-    std::vector<std::string> lines;
-    str::split( vresult, std::back_inserter(lines), "\n" );
+    // Check the individual signature/disgest results:
+
+    // To.map back known result strings to enum, everything else is CHK_ERROR.
+    typedef std::map<std::string_view,RpmDb::CheckPackageResult> ResultMap;
+    static const ResultMap resultMap {
+      { "OK",         RpmDb::CHK_OK },
+      { "NOKEY",      RpmDb::CHK_NOKEY },
+      { "BAD",        RpmDb::CHK_FAIL },
+      { "UNKNOWN",    RpmDb::CHK_NOTFOUND },
+      { "NOTRUSTED",  RpmDb::CHK_NOTTRUSTED },
+      { "NOTFOUND",   RpmDb::CHK_NOTFOUND },
+    };
+    auto getresult = []( const ResultMap & resultMap, ResultMap::key_type key )->ResultMap::mapped_type {
+      auto it = resultMap.find( key );
+      return it != resultMap.end() ? it->second : RpmDb::CHK_ERROR;
+    };
+
+    // To track the signature states we saw.
     unsigned count[7] = { 0, 0, 0, 0, 0, 0, 0 };
 
-    for ( unsigned i = 1; i < lines.size(); ++i )
+    // To track the kind off sigs we saw.
+    enum Saw {
+      SawNone          = 0,
+      SawHeaderSig     = (1 << 0),  // Header V3 RSA/SHA256 Signature, key ID 3dbdc284: OK
+      SawHeaderDigest  = (1 << 1),  // Header SHA1 digest: OK (a60386347863affefef484ff1f26c889373eb094)
+      SawPayloadDigest = (1 << 2),  // Payload SHA256 digest: OK
+      SawSig           = (1 << 3),  // V3 RSA/SHA256 Signature, key ID 3dbdc284: OK
+      SawDigest        = (1 << 4),  // MD5 digest: OK (fd5259fe677a406951dcb2e9d08c4dcc)
+    };
+    unsigned saw = SawNone;
+
+    static const str::regex rx( "^ *(Header|Payload)? .*(Signature, key|digest).*: ([A-Z]+)" );
+    str::smatch what;
+    for ( const std::string & line : vresult )
     {
-      std::string &line( lines[i] );
+      if ( line[0] != ' ' ) // result lines are indented
+        continue;
 
       RpmDb::CheckPackageResult lineres = RpmDb::CHK_ERROR;
-      if ( line.find( ": OK" ) != std::string::npos )
-      {
-        lineres = RpmDb::CHK_OK;
-        if ( line.find( "Signature, key ID" ) == std::string::npos )
-          ++count[RpmDb::CHK_NOSIG];	// Valid but no gpg signature -> CHK_NOSIG
+      if ( str::regex_match( line, what, rx ) ) {
+
+        lineres = getresult( resultMap, what[3] );
+        if ( lineres == RpmDb::CHK_NOTFOUND )
+          continue; // just collect details for signatures found (#229)
+
+        if ( what[1][0] == 'H' ) {
+          saw |= ( what[2][0] == 'S' ? SawHeaderSig :SawHeaderDigest );
+        }
+        else if ( what[1][0] == 'P' ) {
+          if ( what[2][0] == 'd' ) saw |= SawPayloadDigest;
+        }
+        else {
+          saw |= ( what[2][0] == 'S' ? SawSig : SawDigest );
+        }
       }
-      else if ( line.find( ": NOKEY" ) != std::string::npos )
-      { lineres = RpmDb::CHK_NOKEY; }
-      else if ( line.find( ": BAD" ) != std::string::npos )
-      { lineres = RpmDb::CHK_FAIL; }
-      else if ( line.find( ": UNKNOWN" ) != std::string::npos )
-      { lineres = RpmDb::CHK_NOTFOUND; }
-      else if ( line.find( ": NOTRUSTED" ) != std::string::npos )
-      { lineres = RpmDb::CHK_NOTTRUSTED; }
-      else if ( line.find( ": NOTFOUND" ) != std::string::npos )
-      { continue; } // just collect details for signatures found (#229)
 
       ++count[lineres];
       detail_r.push_back( RpmDb::CheckPackageDetail::value_type( lineres, std::move(line) ) );
     }
 
+    // Now combine the overall result:
     RpmDb::CheckPackageResult ret = ( res ? RpmDb::CHK_ERROR : RpmDb::CHK_OK );
 
     if ( count[RpmDb::CHK_FAIL] )
@@ -1228,11 +1260,19 @@ namespace
     else if ( count[RpmDb::CHK_NOTTRUSTED] )
       ret = RpmDb::CHK_NOTTRUSTED;
 
-    else if ( ret == RpmDb::CHK_OK )
-    {
-      if ( count[RpmDb::CHK_OK] == count[RpmDb::CHK_NOSIG]  )
-      {
-        detail_r.push_back( RpmDb::CheckPackageDetail::value_type( RpmDb::CHK_NOSIG, std::string("    ")+_("Package is not signed!") ) );
+    else if ( ret == RpmDb::CHK_OK ) {
+      // Everything is OK, so check whether it's sufficient.
+      // bsc#1184501: To count as signed the package needs a header signature
+      // and either a payload digest (secured by the header sig) or a content signature.
+      bool isSigned = (saw & SawHeaderSig) && ( (saw & SawPayloadDigest) || (saw & SawSig) );
+      if ( not isSigned ) {
+        std::string message { "    " };
+        if ( not (saw & SawHeaderSig) )
+          message += _("Package header is not signed!");
+        else
+          message += _("Package payload is not signed!");
+
+        detail_r.push_back( RpmDb::CheckPackageDetail::value_type( RpmDb::CHK_NOSIG, std::move(message) ) );
         if ( requireGPGSig_r )
           ret = RpmDb::CHK_NOSIG;
       }
@@ -1240,7 +1280,8 @@ namespace
 
     if ( ret != RpmDb::CHK_OK )
     {
-
+      // In case of an error line results may be reported to the user. In case rpm printed
+      // only 8byte key IDs to stdout we try to get longer IDs from the header.
       bool didReadHeader = false;
       std::unordered_map< std::string, std::string> fprs;
 
@@ -1279,7 +1320,7 @@ namespace
             }
           }
 
-          // if we have no keys we can substiture we can leave the loop right away
+          // if we have no keys we can substitute we can leave the loop right away
           if ( !fprs.size() )
             break;
 
@@ -1294,7 +1335,7 @@ namespace
       }
 
       WAR << path_r << " (" << requireGPGSig_r << " -> " << ret << ")" << endl;
-      WAR << vresult;
+      WAR << vresult << endl;
     }
     else
       DBG << path_r << " [0-Signature is OK]" << endl;
@@ -2065,7 +2106,7 @@ bool RpmDb::backupPackage(const std::string& packageName)
 
     std::string tarmsg;
 
-    // TODO: its probably possible to start tar with -v and watch it adding
+    // TODO: it is probably possible to start tar with -v and watch it adding
     // files to report progress
     for (std::string output = tar.receiveLine(); output.length() ;output = tar.receiveLine())
     {
